@@ -3,11 +3,6 @@ from PyObjCTools.TestSupport import TestCase, min_os_level, expectedFailure
 
 
 class TestAVCaptureDevice(TestCase):
-    @expectedFailure
-    def testMissingConstants(self):
-        # Present in header files, but not actually exposed
-        self.assertIsInstance(AVFoundation.AVCaptureMaxAvailableTorchLevel, float)
-
     @min_os_level("10.7")
     def testConstants(self):
         self.assertIsInstance(AVFoundation.AVCaptureDeviceWasConnectedNotification, str)
@@ -54,6 +49,13 @@ class TestAVCaptureDevice(TestCase):
         self.assertEqual(AVFoundation.AVAuthorizationStatusDenied, 2)
         self.assertEqual(AVFoundation.AVAuthorizationStatusAuthorized, 3)
 
+        self.assertEqual(AVFoundation.AVCaptureMicrophoneModeStandard, 0)
+        self.assertEqual(AVFoundation.AVCaptureMicrophoneModeWideSpectrum, 1)
+        self.assertEqual(AVFoundation.AVCaptureMicrophoneModeVoiceIsolation, 2)
+
+        self.assertEqual(AVFoundation.AVCaptureSystemUserInterfaceVideoEffects, 1)
+        self.assertEqual(AVFoundation.AVCaptureSystemUserInterfaceMicrophoneModes, 2)
+
     @min_os_level("10.15")
     def test_constants10_15(self):
         self.assertIsInstance(AVFoundation.AVCaptureDeviceTypeExternalUnknown, str)
@@ -63,6 +65,40 @@ class TestAVCaptureDevice(TestCase):
         )
 
         self.assertIsInstance(AVFoundation.AVCaptureMaxAvailableTorchLevel, float)
+        self.assertIsInstance(AVFoundation.AVCaptureMaxAvailableTorchLevel, float)
+
+    @min_os_level("12.0")
+    def test_constants12_0(self):
+        self.assertEqual(
+            AVFoundation.AVCapturePrimaryConstituentDeviceSwitchingBehaviorUnsupported,
+            0,
+        )
+        self.assertEqual(
+            AVFoundation.AVCapturePrimaryConstituentDeviceSwitchingBehaviorAuto, 1
+        )
+        self.assertEqual(
+            AVFoundation.AVCapturePrimaryConstituentDeviceSwitchingBehaviorRestricted, 2
+        )
+        self.assertEqual(
+            AVFoundation.AVCapturePrimaryConstituentDeviceSwitchingBehaviorLocked, 3
+        )
+
+        self.assertEqual(
+            AVFoundation.AVCapturePrimaryConstituentDeviceRestrictedSwitchingBehaviorConditionNone,
+            0,
+        )
+        self.assertEqual(
+            AVFoundation.AVCapturePrimaryConstituentDeviceRestrictedSwitchingBehaviorConditionVideoZoomChanged,
+            1 << 0,
+        )
+        self.assertEqual(
+            AVFoundation.AVCapturePrimaryConstituentDeviceRestrictedSwitchingBehaviorConditionFocusModeChanged,
+            1 << 1,
+        )
+        self.assertEqual(
+            AVFoundation.AVCapturePrimaryConstituentDeviceRestrictedSwitchingBehaviorConditionExposureModeChanged,
+            1 << 2,
+        )
 
     @min_os_level("10.7")
     def testMethods(self):
@@ -97,6 +133,10 @@ class TestAVCaptureDevice(TestCase):
             AVFoundation.AVCaptureDevice.isWhiteBalanceModeSupported_
         )
         self.assertResultIsBOOL(AVFoundation.AVCaptureDevice.isAdjustingWhiteBalance)
+
+    @expectedFailure  # XXX
+    @min_os_level("10.7")
+    def testMethods_error_on_11(self):
         self.assertResultIsBOOL(AVFoundation.AVCaptureDevice.transportControlsSupported)
 
     @min_os_level("10.14")
@@ -119,3 +159,17 @@ class TestAVCaptureDevice(TestCase):
         self.assertArgIsOut(
             AVFoundation.AVCaptureDevice.setTorchModeOnWithLevel_error_, 1
         )
+
+    @min_os_level("12.0")
+    def testMethods12_0(self):
+        self.assertResultIsBOOL(
+            AVFoundation.AVCaptureDeviceFormat.isHighPhotoQualitySupported
+        )
+
+        self.assertResultIsBOOL(AVFoundation.AVCaptureDevice.isPortraitEffectEnabled)
+
+        self.assertResultIsBOOL(AVFoundation.AVCaptureDevice.isPortraitEffectActive)
+
+        self.assertResultIsBOOL(
+            AVFoundation.AVCaptureDeviceFormat.isPortraitEffectSupported
+        )
